"""Meta package for the Ubermag project."""
import importlib.metadata

import pytest

from .info import debug_info

__version__ = importlib.metadata.version(__package__)


def test():
    """Run all package tests.

    Examples
    --------
    1. Run all tests.

    >>> import ubermag
    ...
    >>> # ubermag.test()

    """
    return pytest.main(
        [
            "-m",
            "not travis and not docker",
            "-v",
            "--pyargs",
            "ubermagutil",
            "discretisedfield",
            "ubermagtable",
            "micromagneticmodel",
            "oommfc",
            "micromagnetictests",
            "micromagneticdata",
            "mag2exp",
            # "mumax3c",  # mumax3 is not necessarily available -> no tests
            "ubermag",
            "-l",
        ]
    )  # pragma: no cover



import importlib
import sys


def debug_info():
    """Collect inforamtion about ubermag subpackages and available calculators."""
    info_str = f"Platform: {sys.platform}\nPython: {sys.version}\n\n"

    for pkg in [
        "ubermagutil",
        "discretisedfield",
        "ubermagtable",
        "micromagneticmodel",
        "micromagneticdata",
        "micromagnetictests",
        "mumax3c",
        "oommfc",
        "mag2exp",
        "ubermag",
    ]:
        module = importlib.import_module(pkg)
        version = getattr(module, "__version__")
        info_str += f"{pkg}: {version}\n"

    info_str += "\n"

    for pkg, calculator in [("oommfc", "OOMMF"), ("mumax3c", "Mumax3")]:
        try:
            module = importlib.import_module(pkg)
            runner = getattr(module, "runner").__getattribute__("runner")
        except OSError as e:
            info_str += f"{calculator}: {e}\n"
        else:
            info_str += f"{calculator}: {runner}, {runner.version}\n"

    return info_str



import re

import ubermag


def test_debug_info():
    info = ubermag.debug_info()

    assert "Python: 3." in info

    for pkg in [
        "ubermagutil",
        "discretisedfield",
        "ubermagtable",
        "micromagneticmodel",
        "micromagneticdata",
        "micromagnetictests",
        "mumax3c",
        "oommfc",
        "mag2exp",
    ]:
        # test that all packages are listed with some version number
        assert re.findall(rf"{pkg}: [0-9.]+", info)

    assert f"ubermag: {ubermag.__version__}" in info

    # OOMMF is automatically installed so we can expect to find a Runner
    assert "OOMMFRunner" in info
